from functools import wraps
from xmlrpc.client import Fault as SupervisorFault

from tfw.event_handler_base import TriggerlessEventHandler
from tfw.components.mixins import SupervisorMixin
from tfw.config.logs import logging
log = logging.getLogger(__name__)


class ProcessManager(SupervisorMixin):
    def __init__(self):
        self.commands = {'start': self.start_process,
                         'stop': self.stop_process,
                         'restart': self.restart_process,
                         'readlog': self.read_log}

    def __call__(self, command, process_name):
        return self.commands[command](process_name)


class ProcessManagingEventHandler(TriggerlessEventHandler):
    def __init__(self, key, dirmonitor=None):
        super().__init__(key)
        self.key = key
        self.monitor = dirmonitor
        self.processmanager = ProcessManager()

    def _with_monitor_paused(fun):
        @wraps(fun)
        def wrapper(self, *args, **kwargs):
            if self.monitor:
                with self.monitor.pauser:
                    return fun(self, *args, **kwargs)
            return fun(self, *args, **kwargs)
        return wrapper

    @_with_monitor_paused
    def handle_event(self, key, data_json):
        try:
            data = data_json['data']
            self.processmanager(data['command'], data['process_name'])
            return data_json
        except KeyError:
            log.error('IGNORING MESSAGE: Invalid data_json received: {}'.format(data_json))
        except SupervisorFault as fault:
            data_json['data']['error'] = fault.faultString
            data_json['data']['log'] = self.processmanager.read_log(data_json['data']['process_name'])
            log.debug('Failed to execute command on process: {}'.format(data_json))
            return data_json
